
clc
close all
%clear all

%%% Simulation of a two-link manipulator
% <<two_link.png>>

%%%%%%%%% INITIALIZE PARAMETERS %%%%%%
%DH for link 1 except theta1
parms.a1   =  1;            
parms.alpha1 = 0; 
parms.d1=0;

%DH for link 2 except theta2
parms.a2   =  1;            
parms.alpha2 = 0; 
parms.d2=0;

%mass, inertia and gravity.
parms.m1  =  1;   
parms.I1  =  0.5; 
parms.m2  =  1;   
parms.I2  =  0.5; 
parms.g   =  10;

%stuff for animation
parms.time_delay = 0.001; %delay between frames, will need some fine tuning for different computers
parms.framespersec = 15;

%step size for integration. Accuracy increases as h decreases
h = 0.001;

%set the time
t0 = 0;
tN = 10;
N = (tN-t0)/h;
t = linspace(t0,tN,N);

%initial conditions
theta1    = pi/2-0.01; 
theta1dot    = 0;  
theta2    = 0; 
theta2dot    = 0;  
x0=[theta1 theta1dot theta2 theta2dot]';


%integrate equations of motion
x = ode4('twolink_rhs',t,x0,parms);

%Do some plots
figure(1)
plot(t,x(:,1),'b',t,x(:,3),'r'); 
xlabel('time');
ylabel('position');
legend('link1','link2');

figure(2)
plot(t,x(:,2),'b',t,x(:,4),'r');
xlabel('time');
ylabel('velocity');
legend('link1','link2');

figure(3)  %animation
twolink_animation(t,[x(:,1), x(:,3)],parms);

